@@ -81,10 +81,10 @@ enmtools.maxent <- function(species, env, test.prop = 0, nback = 1000, env.nback
   }
 
   if(verbose){
-    this.mx <- dismo::maxent(env, p = analysis.df[analysis.df$presence == 1,1:2], a = analysis.df[analysis.df$presence == 0,1:2], ...)
+    this.mx <- dismo::maxent(raster::stack(env), p = analysis.df[analysis.df$presence == 1,1:2], a = analysis.df[analysis.df$presence == 0,1:2], ...)
     suitability <- terra::predict(env, this.mx, type = "response", na.rm = TRUE, ...)
   } else {
-    invisible(capture.output(this.mx <- dismo::maxent(env, p = analysis.df[analysis.df$presence == 1,1:2], a = analysis.df[analysis.df$presence == 0,1:2], ...)))
+    invisible(capture.output(this.mx <- dismo::maxent(raster::stack(env), p = analysis.df[analysis.df$presence == 1,1:2], a = analysis.df[analysis.df$presence == 0,1:2], ...)))
     invisible(capture.output(suitability <- terra::predict(env, this.mx, type = "response", na.rm = TRUE, ...)))
   }
 
@@ -222,12 +222,12 @@ enmtools.maxent <- function(species, env, test.prop = 0, nback = 1000, env.nback
 
       # We have to do this to capture the "this is maxent version XXX message".
       if(verbose){
-        thisrep.mx <- dismo::maxent(env, p = rts.df[rts.df$presence == 1,1:2], a = rts.df[rts.df$presence == 0,1:2], ...)
+        thisrep.mx <- dismo::maxent(raster::stack(env), p = rts.df[rts.df$presence == 1,1:2], a = rts.df[rts.df$presence == 0,1:2], ...)
         thisrep.model.evaluation <- dismo::evaluate(rep.species$presence.points, species$background.points,
                                                    thisrep.mx, env, na.rm = TRUE)
         thisrep.env.model.evaluation <- env.evaluate(rep.species, thisrep.mx, env, n.background = env.nback)
       } else {
-        invisible(capture.output(thisrep.mx <- dismo::maxent(env, p = rts.df[rts.df$presence == 1,1:2], a = rts.df[rts.df$presence == 0,1:2], ...)))
+        invisible(capture.output(thisrep.mx <- dismo::maxent(raster::stack(env), p = rts.df[rts.df$presence == 1,1:2], a = rts.df[rts.df$presence == 0,1:2], ...)))
         invisible(capture.output(thisrep.model.evaluation <- dismo::evaluate(rep.species$presence.points, species$background.points,
                                                    thisrep.mx, env, na.rm = TRUE)))
         invisible(capture.output(thisrep.env.model.evaluation <- env.evaluate(rep.species, thisrep.mx, env, n.background = env.nback)))
